package org.xbib.interlibrary.client.gbv.order;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.xbib.interlibrary.api.order.tracking.Event;
import org.xbib.interlibrary.api.order.tracking.EventHandler;
import org.xbib.interlibrary.http.HttpHelper;
import org.xbib.settings.Settings;

public class OrderTracker implements org.xbib.interlibrary.api.order.tracking.OrderTracker {

    private static final Map<String, String> domainMap = new HashMap<>() {{
        put("DE-576", "SWB");
        put("DE-601", "GBV");
        put("DE-602", "KOBV");
        put("DE-603", "HEBIS");
        put("DE-604", "BVB");
        put("DE-605", "HBZ");
    }};

    private static final Map<String, String> codeMap = new HashMap<>() {{
        put("1", "Original ist auf dem Weg zu Ihrer Bibliothek");
        put("2", "Kopie ist auf dem Weg zu Ihrer Bibliothek");
        put("3", "Zugeschickt");
        put("4", "Noch nicht vorhanden");
        put("5", "Nicht vorhanden");
        put("6", "Nicht verfügbar");
        put("7", "Ausgeliehen");
        put("8", "Nicht ausleihbar");
        put("9", "Bibliographisch nicht korrekt");
        put("0", "Beim Buchbinder");
        put("H", "Bestellung weitergeleitet");
        put("P", "Bestellung weitergegeben");
        put("S", "Bestellung storniert");
    }};

    private static final Pattern pattern =
            Pattern.compile("<TR><TD>(.*?)</TD><TD>(.*?)</TD><TD>(.*?)</TD><TD>(.*?)</TD><TD>(.*?)</TD><TD>(.*?)</TD></TR>",
                    Pattern.MULTILINE);

    private static final DateTimeFormatter formatter = new DateTimeFormatterBuilder()
            .parseCaseInsensitive()
            .appendPattern("dd.MM.yyyy")
            .toFormatter(Locale.GERMAN);

    private HttpHelper httpHelper;

    private String urlPatternOne;

    private String urlPatternTwo;

    public OrderTracker() {
    }

    @Override
    public void init(Settings settings) throws IOException {
        this.httpHelper = new HttpHelper();
        httpHelper.open(settings);
        // A-Nummer
        this.urlPatternOne = settings.get("url.singleparam");
        Objects.requireNonNull(urlPatternOne);
        // "HBZ" + TAN
        this.urlPatternTwo = settings.get("url.doubleparam");
        Objects.requireNonNull(urlPatternTwo);
    }

    @Override
    public String getISIL() {
        return "DE-601";
    }

    @Override
    public void track(String orderNumber, EventHandler eventHandler) throws IOException {
        String url = String.format(urlPatternOne, orderNumber.trim());
        httpHelper.get(url,
                params -> {},
                headers -> {},
                httpResponse -> {
                    String body = httpResponse.getBodyAsChars(StandardCharsets.UTF_8).toString();
                    parse(body, eventHandler);
                },
                exception -> {
                    throw new IllegalStateException(exception);
                },
                req -> {
                });
    }

    @Override
    public void track(String domain, String requesterId, String transactionId, EventHandler eventHandler) throws IOException {
        String mappedDomain = domainMap.getOrDefault(domain, domain);
        String url = String.format(urlPatternTwo, mappedDomain, transactionId.trim());
        httpHelper.get(url,
                params -> {},
                headers -> {},
                httpResponse -> {
                    String body = httpResponse.getBodyAsChars(StandardCharsets.UTF_8).toString();
                    parse(body, eventHandler);
                },
                exception -> {
            throw new IllegalStateException(exception);
                },
                req -> {
                });
    }

    private void parse(String body, EventHandler eventHandler) {
        final Matcher matcher = pattern.matcher(body);
        while (matcher.find()) {
            final Event event = new Event() {

                @Override
                public Integer getPos() {
                    try {
                        return Integer.parseInt(matcher.group(1));
                    } catch (Exception e) {
                        return null;
                    }
                }

                @Override
                public LocalDateTime getTime() {
                    String time = matcher.group(4);
                    if (!time.isEmpty() && !time.isBlank()) {
                        LocalDate localDate = LocalDate.parse(time, formatter);
                        return localDate.atTime(LocalTime.MIN);
                    } else {
                        return null;
                    }
                }

                @Override
                public String getIsil() {
                    return matcher.group(2);
                }

                @Override
                public String getCode() {
                    return codeMap.get(matcher.group(5));
                }

                @Override
                public String getInfo() {
                    return matcher.group(6);
                }
            };
            eventHandler.onEvent(event);
        }
    }
}
