package org.xbib.interlibrary.client.gbv.sru;

import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.function.Consumer;
import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.config.Configuration;
import org.xbib.interlibrary.api.library.Library;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.library.DefaultLibraryService;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.api.library.Mode;
import org.xbib.interlibrary.sru.AbstractSRUAvailService;
import org.xbib.interlibrary.sru.DisplayHelper;
import org.xbib.interlibrary.sru.SRUClientHelper;
import org.xbib.marc.MarcRecord;
import org.xbib.marc.MarcRecordListener;
import static org.xbib.interlibrary.api.library.Material.ONLINE_RESOURCE;

public class SRUAvailService extends AbstractSRUAvailService {

    public SRUAvailService(ServiceArguments arguments, SRUClientHelper sruClientHelper) {
        super(arguments, sruClientHelper);
    }

    @Override
    protected String supports(String source) {
        return source != null && source.toLowerCase(Locale.ROOT).contains("gbv") ? "GBV" : null;
    }

    @Override
    protected MarcRecordListener getMarcRecordListener(AvailRequest availRequest,
                                                       AvailServiceResponseBuilder availResponse,
                                                       MultiMap<String, LibraryService> multiMap) {
        return new MarcRecordListener() {

            @Override
            public void beginCollection() {
            }

            @Override
            public void record(MarcRecord marcRecord) {
                if (availRequest.isBibliographicDescriptionEnabled()) {
                    availResponse.setBibliographicDescription(marcRecord);
                    availResponse.setDisplay(DisplayHelper.display(marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                            marcHelper.getRecordIdSupplier(marcRecord)));
                }
                for (LibraryService service : extractGermanInterlibraryServices(availRequest, marcRecord)) {
                    multiMap.put(service.getISIL(), service);
                }
            }

            @Override
            public void endCollection() {
            }
        };
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }

    @Override
    protected String sourceIdCQL(String sourceId) {
        return "pica.ppn = " + sourceId;
    }

    @Override
    protected Consumer<TinyMap.Builder<String, Object>> parameterCompletion() {
        return params -> params.put("maximumRecords", 1);
    }

    private void extractGbvInterlibraryServices(Configuration interlibraryConfiguration,
                                                AvailRequest availRequest,
                                                MarcRecord marcRecord,
                                                Collection<String> fields,
                                                MultiMap<String, LibraryService> multiMap) {
        List<Material> materials = marcHelper.extractMaterial(marcRecord);
        marcRecord.all(marcField -> fields.contains(marcField.getTag()), marcField -> {
            LibraryService.Builder libraryServiceBuilder = DefaultLibraryService.builder();
            libraryServiceBuilder.setId(Integer.toString(random()));
            DefaultLibraryService.DefaultLocation location = new DefaultLibraryService.DefaultLocation();
            DefaultLibraryService.DefaultLabeledUrl labeledUrl = new DefaultLibraryService.DefaultLabeledUrl();
            StringBuilder sb = new StringBuilder();
            marcField.getSubfields().forEach(subfield -> {
                switch (subfield.getId()) {
                    case "9":
                        // sigel, skip
                        break;
                    case "b":
                        // ISIL
                        String isil = subfield.getValue();
                        sb.append(isil);
                        Library library = interlibraryConfiguration.getLibrary(isil);
                        if (library != null) {
                            libraryServiceBuilder.setISIL(library.getISIL());
                            libraryServiceBuilder.setName(library.getName());
                            libraryServiceBuilder.setDomainPriority(domainPriority(availRequest, library));
                            libraryServiceBuilder.setLibraryPriority(libraryPriority(availRequest, library));
                            libraryServiceBuilder.setMaterials(materials);
                            libraryServiceBuilder.setServicePriority(materials.contains(ONLINE_RESOURCE) ? 1 : 2);
                        }
                        break;
                    case "d":
                        // call number
                        location.setCallnumber(subfield.getValue());
                        break;
                    case "f":
                        // public note
                        location.setDescription(subfield.getValue());
                        break;
                    case "k":
                        // URL
                        labeledUrl.setUrl(subfield.getValue());
                        break;
                    case "l":
                        // URL label
                        labeledUrl.setLabel(subfield.getValue());
                        break;
                    case "z":
                        // loan indicator
                        switch (subfield.getValue()) {
                            case "L", "a" -> libraryServiceBuilder.setMode(Mode.LOAN);
                            case "C", "b" -> libraryServiceBuilder.setMode(Mode.COPY);
                            case "LC", "c" -> libraryServiceBuilder.setMode(Mode.LOAN_COPY);
                            case "N", "d" -> libraryServiceBuilder.setMode(Mode.NONE);
                            case "e" -> libraryServiceBuilder.setMode(Mode.ELECTRONIC_COPY);
                            default -> {
                            }
                        }
                        break;
                    default:
                        break;
                }
            });
            if (labeledUrl.getUrl() != null) {
                location.addLink(labeledUrl);
            }
            if (!sb.isEmpty()) {
                libraryServiceBuilder.addLocation(location);
                multiMap.put(sb.toString(), libraryServiceBuilder.build());
            }
        });
    }
}
