package org.xbib.interlibrary.client.gbv.z;

import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.library.Library;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.library.DefaultLibraryService;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.api.library.Mode;
import org.xbib.interlibrary.z.AbstractZAvailService;
import org.xbib.interlibrary.z.DisplayHelper;
import org.xbib.marc.MarcRecord;

import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;
import static org.xbib.interlibrary.api.library.Material.ONLINE_RESOURCE;

public class ZAvailService extends AbstractZAvailService {

    private static final Logger logger = Logger.getLogger(ZAvailService.class.getName());

    public ZAvailService(ServiceArguments arguments) {
        super(arguments);
    }

    @Override
    protected String supports(String source) {
        return source.toLowerCase(Locale.ROOT).contains("gbv") ? "GBV" : null;
    }

    @Override
    protected void avail(AvailRequest availRequest,
                         AvailServiceResponseBuilder availResponse,
                         MultiMap<String, LibraryService> multiMap,
                         MarcRecord marcRecord) {
        boolean found = false;
        if (availRequest.isBibliographicDescriptionEnabled()) {
            availResponse.setBibliographicDescription(marcRecord);
            TinyMap.Builder<String, Object> builder = TinyMap.builder();
            DisplayHelper.display(builder, marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                    marcHelper.getRecordIdSupplier(marcRecord));
            availResponse.setDisplay(builder.build());
        }
        for (LibraryService service : extractGermanInterlibraryServices(availRequest, marcRecord)) {
            String isil = service.getISIL();
            multiMap.put(isil, service);
            found = true;
        }
        if (found) {
            availResponse.setStatus(getName(), Response.Status.OK);
        } else {
            availResponse.setStatus(getName(), Response.Status.EMPTY_RESPONSE);
        }
    }

    protected List<LibraryService> extractGbvServices(AvailRequest availRequest,
                                                      MarcRecord marcRecord) {
        final List<LibraryService> list = new LinkedList<>();
        String identifier = identifier(marcRecord);
        List<Material> materials = marcHelper.extractMaterial(marcRecord);
        marcRecord.getAll("900").forEach(marcField -> {
            DefaultLibraryService.Builder serviceBuilder = DefaultLibraryService.builder();
            serviceBuilder.setId(Integer.toString(random()));
            serviceBuilder.setMode(Mode.LOAN_COPY);
            DefaultLibraryService.DefaultLocation location = new DefaultLibraryService.DefaultLocation();
            marcField.getSubfields().forEach(subfield -> {
                switch (subfield.getId()) {
                    case "a" -> {
                        // always "GBV"
                    }
                    case "b" -> {
                        // library name <Sigel>
                        String sigel = extractSigel(subfield.getValue());
                        Library library = configuration.getLibrary(sigel);
                        if (library != null) {
                            serviceBuilder.setISIL(library.getISIL());
                            serviceBuilder.setName(library.getName());
                            serviceBuilder.setDomainPriority(domainPriority(availRequest, library));
                            serviceBuilder.setLibraryPriority(libraryPriority(availRequest, library));
                            serviceBuilder.setMaterials(materials);
                            serviceBuilder.setServicePriority(materials.contains(ONLINE_RESOURCE) ? 1 : 2);
                            serviceBuilder.setSource(library.getDomain().getOrganization());
                            serviceBuilder.setSourceId(identifier);
                        } else {
                            logger.log(Level.WARNING, () -> "no ISIL for: " + subfield.getValue());
                        }
                    }
                    case "c" -> {
                        // description
                        location.setDescription(subfield.getValue());
                    }
                    case "d" -> {
                        // callnumber
                        location.setCallnumber(subfield.getValue());
                    }
                    case "z" -> {
                        // loan indicator - but only for e-resources?
                        switch (subfield.getValue()) {
                            case "L", "a" -> serviceBuilder.setMode(Mode.LOAN);
                            case "C", "b" -> serviceBuilder.setMode(Mode.COPY);
                            case "LC", "c" -> serviceBuilder.setMode(Mode.LOAN_COPY);
                            case "N", "d" -> serviceBuilder.setMode(Mode.NONE);
                            case "e" -> serviceBuilder.setMode(Mode.ELECTRONIC_COPY);
                            default -> {
                                logger.log(Level.WARNING, "unrecognized service value " + subfield.getValue());
                            }
                        }
                    }
                    default -> {
                        logger.log(Level.WARNING, "unrecognized subfield " + subfield);
                    }
                }
            });
            serviceBuilder.addLocation(location);
            list.add(serviceBuilder.build());
        });
        return list;
    }

    private String extractSigel(String name) {
        int pos1 = name.indexOf('<');
        int pos2 = name.indexOf('>');
        if (pos1 > 0 && pos2 > 0) {
            return name.substring(pos1 + 1, pos2);
        } else {
            return name;
        }
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
